<?php

namespace Andy\ChangeAuthor\XF\Pub\Controller;

use XF\Mvc\ParameterBag;

class Post extends XFCP_Post
{
	public function actionChangeAuthor(ParameterBag $params)
	{
		// get visitor
		$visitor = \XF::visitor();
				
		// get permission
		if (!$visitor->hasPermission('changeAuthor', 'view'))
		{
			return $this->noPermission();
		}
		
		// get post entity
		$post = $this->assertViewablePost($params->post_id);	

		// prepare viewParams
		$viewParams = [
			'post' => $post
		];
		
		// send to template
		return $this->view('XF:Andy\ChangeAuthor', 'andy_changeauthor', $viewParams);
	}
	
	public function actionChangeAuthorSave(ParameterBag $params)
	{	
		// get visitor
		$visitor = \XF::visitor();
				
		// get permission
		if (!$visitor->hasPermission('changeAuthor', 'view'))
		{
			return $this->noPermission();
		}		
		
		// get post entity
		$post = $this->assertViewablePost($params->post_id);		
		
		// assert post only
		$this->assertPostOnly();
		
		// get newPostAuthor
		$newPostAuthor = $this->filter('new_post_author', 'str');
		
		// get postId
		$postId = $post->post_id;
		
		// get threadId
		$threadId = $post->thread_id;				
		
		// get position
		$position = $post->position;	
		
		//########################################
		// start db operations				
		
		// get db
		$db = \XF::db();	
		
		// run query
		$result = $db->fetchRow("
		SELECT user_id,
		username
		FROM xf_user
		WHERE username = ?
		", $newPostAuthor);	
		
		// get newUserId
		$newUserId = $result['user_id'];	
		
		// if userId not found
		if ($newUserId == '')
		{
			return $this->error(\XF::phrase('requested_user_not_found'));
		}		
		
		// get newPostAuthor (corrects any case issues)
		$newPostAuthor = $result['username'];	
		
		// run query
		$oldUserId = $db->fetchOne('
		SELECT user_id
		FROM xf_post
		WHERE post_id = ?
		', $postId);	
		
		//########################################
		// update xf_post and xf_thread					
		
		// run query
		$db->query("
			UPDATE xf_post SET
				user_id = ?, 
				username = ?
			WHERE post_id = ?
		", array($newUserId, $newPostAuthor, $postId));	
		
		// update if first post of thread
		if ($position == 0)
		{
			// run query
			$db->query("
				UPDATE xf_thread SET
					user_id = ?, 
					username = ?
				WHERE thread_id = ?
			", array($newUserId, $newPostAuthor, $threadId));
		}
		
		//########################################
		// update xf_forum and xf_thread
		// last post data

		// run query
		$nodeId = $db->fetchOne("
		SELECT node_id
		FROM xf_thread
		WHERE last_post_id = ?
		", $postId);
		
		// continue if last post
		if ($nodeId != '')
		{		
			// run query
			$db->query("
				UPDATE xf_forum SET
					last_post_user_id = ?,
					last_post_username = ?
				WHERE node_id = ?
			", array($newUserId, $newPostAuthor, $nodeId));				
	
			// run query
			$db->query("
				UPDATE xf_thread SET
					last_post_user_id = ?,
					last_post_username = ?
				WHERE thread_id = ?
			", array($newUserId, $newPostAuthor, $threadId));
		}
		
		//########################################
		// update xf_thread_user_post	
		
		// run query
		$db->query("
			UPDATE xf_thread_user_post SET
				post_count = post_count - 1
			WHERE thread_id = ?
			AND user_id = ?
			AND post_count > ?
		", array($threadId, $oldUserId, 0));
		
		// run query
		$db->query("
			INSERT INTO xf_thread_user_post
				(thread_id, user_id, post_count)
			VALUES 
				(?,?,?)
			ON DUPLICATE KEY UPDATE
				post_count = post_count + 1
		", array($threadId, $newUserId, 1));	

		//########################################
		// update xf_user.message_count
		
		// check if messages are counted in this forum
		$countMessages = $db->fetchOne("
		SELECT xf_forum.count_messages
		FROM xf_post
		INNER JOIN xf_thread ON xf_thread.thread_id = xf_post.thread_id
		INNER JOIN xf_forum ON xf_forum.node_id = xf_thread.node_id
		WHERE xf_post.post_id = ?
		", $postId);
		
		// continue if countMessages
		if ($countMessages)
		{
			// run query
			$db->query("
				UPDATE xf_user
				SET message_count = message_count - 1
				WHERE user_id = ?
				AND message_count > ?
			", array($oldUserId, 0));	
			
			// run query
			$db->query("
				UPDATE xf_user
				SET message_count = message_count + 1
				WHERE user_id = ?
			", $newUserId);
		}
		
		//########################################
		// update xf_search_index
		
		if ($position == 0)
		{
			// run query
			$db->query("
				UPDATE xf_search_index
				SET user_id = ?
				WHERE content_type = ?
				AND content_id = ?
			", array($newUserId, 'thread', $threadId));
		}
		
		if ($position > 0)
		{
			// run query
			$db->query("
				UPDATE xf_search_index
				SET user_id = ?
				WHERE content_type = ?
				AND content_id = ?
			", array($newUserId, 'post', $postId));
		}
		
		//########################################
		// return redirect						

		return $this->redirect(
			$this->getDynamicRedirect($this->buildLink('posts', $post), false)
		);
	}
}